/*
 * ServiceWrapper.java
 * Created on 5th July 11:40
 */

import school.*;
import java.io.*;
/**
 * Class for wrapping the calls to the services 
 * RandGen, SeqGen, XProdGen, Surface1, Visualiser and Converter
 * @author sara
 */
public class ServiceWrapper{

    /** Generates a random set of 2D data (x,y) pairs which is
     * stored in FileStore
     * @param boundary the bounding box defining the ranges for the
     * random pairs.
     * @param count the number of pairs to be generated
     * @param name the name of the file in which to store the data
     * @param dataDir the sub-directory in FileStore where the
     * file will be put.
     */
    public static void randgen(Box boundary, int count, String name, 
			     String dataDir){
	try{
	    //file and directory for random 2D data
	    FileName2D fileName = new FileName2D(dataDir,name);

	    //generate the random 2D data which is automatically
	    //stored using FileStore
	    RandGen rg = new RandGen();
	    rg.generate(boundary,count,fileName);
	}
	catch(GeneratorException ge){
	    System.err.println(ge.getMessage());
	    System.exit(1);
	}
    }

    /** Generates a sequence of real numbers which is
     * stored in FileStore
     * @param start the start value for the sequence
     * @param step the spacing between values
     * @param limit the maximum value in the sequence
     * @param seqName the name of the file in which to store the data
     * @param seqDir the sub-directory in FileStore where the
     * file will be put.
     */
    public static void seqgen(double start, double step,
			    double limit, String seqName,
			    String seqDir){
	try{
	    //file and directory for 1D data
	    FileName1D fileSeq = new FileName1D(seqDir,seqName);

	    //generate the sequence which is automatically
	    //stored using FileStore
	    SeqGen sg = new SeqGen();
	    sg.generate(start,step,limit,fileSeq);

	}
	catch(GeneratorException ge){
	    System.err.println(ge.getMessage());
	    System.exit(1);
	}
    }
    
    /** Generates the cross-product of two 1D sequences of
     * real numbers. 
     * @param seq1Name the filename of the first sequence
     * @param seq1Dir the subdirectory in FileStore where the
     * first sequence is stored
     * @param seq2Name the filename of the second sequence
     * @param seq2Dir the subdirectory in FileStore where the
     * second sequence is stored
     * @param xprodName the filename for the cross-product data
     * @param xprodDir the subdirectory in FileStore in which to store
     * the data
     */
    public static void xprodgen(String seq1Name, String seq1Dir,
				     String seq2Name, String seq2Dir,
				     String xprodName, String xprodDir){
	try{
	    //filenames and directory for sequence data
	    FileName1D fileSeq1 = new FileName1D(seq1Dir,seq1Name);
	    FileName1D fileSeq2 = new FileName1D(seq2Dir,seq2Name);

	    //filenames and directory for cross-product 2D data 
	    FileName2D fileXprod = new FileName2D(xprodDir,xprodName);

	    //generate the cross-product which is then automatically
	    //store in FileStore
	    XProdGen xp = new XProdGen();
	    xp.generate(fileSeq1,fileSeq2,fileXprod);
	}
	catch(GeneratorException ge){
	    System.err.println(ge.getMessage());
	    System.exit(1);
	}
    }

    /** Samples a predefined surface generating 3D data
     * (x,y,h) from 2D (x,y) input data
     * @param inputName the 2D input file
     * @param inputDir the sub-directory of FileStore where
     * the 2D data is stored
     * @param surfName the filename of the 3D output data
     * @param surfDir the sub-directory of FileStore to store the
     * 3D data in.
    */
    public static void probe(String inputName, String inputDir,
				     String surfName, String surfDir){
	try{
	    //filename and directory for the input data
	    FileName2D fileName = new FileName2D(inputDir,inputName);

	    //filename and directory for the sampled surface
	    FileName3D out = new FileName3D(surfDir,surfName);

	    //sample the surface.
	    Surface1 s = new Surface1();
	    s.probe(fileName,out);
	}
	catch(ProbeException pe){
	    System.err.println(pe.getMessage());
	    System.exit(1);
	}
    }

    /** Visualises the 3D sampled surface using gnuplot to generate
     * postscript which is outputted to a file
     * @param surfName the filename for the 3D data
     * @param surfDir the sub-directory in FileStore where it is
     * stored
     * @param epsName the filename for the postscript file which
     * will be produced in the directory the program is run in.
     */
    public static void render(String surfName, String surfDir,
				 String epsName){
	try{
	    //filename and directory for the 3-D data
	    FileName3D f =  new FileName3D(surfDir,surfName);

	    //produce the postscript using render and output
	    //to a file
	    Visualiser v = new Visualiser();
	    String out = v.render(f);
	    outputToFile(out,epsName);
	}
	catch(RenderException re){
	    System.err.println(re.getMessage());
	    System.exit(1);
	}
    }

    /** Converts the 3D sampled surface into the format required for
     * inputting to gnuplot
     * @param surfName the filename for the 3D data
     * @param surfDir the sub-directory in FileStore where it is
     * stored
     * @param gnuName the filename for the gnu formatted file which
     * will be produced in the directory the program is run in.
     */
    public static void reformat(String surfName, String surfDir,
				 String gnuName){
	try{
	    //filename and directory for the 3-D data
	    FileName3D f =  new FileName3D(surfDir,surfName);

	    //convert the data to gnu format and output to file
	    Reformat v = new Reformat();
	    String out = v.reformat(f);
	    outputToFile(out,gnuName);
	}
	catch(ReformatException re){
	    System.err.println(re.getMessage());
	    System.exit(1);
	}
    }


    /** Writes data into a file in the local directory
     * @param out the data to be written
     * @param name the filename
     */
    private static void outputToFile(String out, String name){
	try{
	    File outFile = new File(name);
	    if(!outFile.createNewFile()){
		System.err.println("Cannot create file "+name+
				   " it might already exist");
		System.exit(1);
	    }
	    PrintWriter pw = new PrintWriter(new FileWriter(outFile));
	    pw.println(out);
	    pw.close();
	}
	catch(IOException ioe){
	    System.err.println("Error writing to file "+name);
	    System.exit(1);
	}

    }
}
